require "save"

local warp_buf = am.load_audio("audio/warp.ogg")
local gameover_buf = am.load_audio("audio/gameover.ogg")
local land_buf = am.load_audio("audio/land.ogg")
local jump_buf = am.load_audio("audio/jump.ogg")
local collect_buf = am.load_audio("audio/collect.ogg")
local portal_open_buf = am.load_audio("audio/portal_open.ogg")

local curr_track = nil

local stage_tracks = {
    "audio/stage1.ogg",
    "audio/stage2.ogg",
    "audio/stage3.ogg",
    "audio/stage4.ogg",
}

local boss_tracks = {
    "audio/boss.ogg",
    "audio/boss.ogg",
    "audio/boss.ogg",
    "audio/boss.ogg",
}

function start_end_stage_music()
    if not global_state.music_muted then
        curr_track = "audio/endstage.ogg"
        local buf = am.load_buffer(curr_track)
        local stream = am.stream(buf, true)
        root_scene:action("music", am.play(stream))
    end
end

function start_music(level_num, nobosses)
    if not global_state.music_muted then
        local tracks = stage_tracks
        if not nobosses and level_is_boss(level_num) then
            tracks = boss_tracks
        end
        local stage = get_level_stage(level_num)
        local track_name = tracks[(stage - 1) % #tracks + 1]
        if track_name ~= curr_track then
            local buf = am.load_buffer(track_name)
            local stream = am.stream(buf, true)
            root_scene:action("music", am.play(stream))
            curr_track = track_name
        end
    end
end

function start_menu_music()
    if global_state.saved_game then
        start_music(global_state.saved_game.level, true)
    else
        start_music(1)
    end
end

function stop_music()
    root_scene:cancel("music")
    curr_track = nil
end

function play_audio_warp()
    if not global_state.sfx_muted then
        root_scene:action(am.play(warp_buf, false, 1))
    end
end

function play_audio_gameover()
    if not global_state.sfx_muted then
        root_scene:action(am.play(gameover_buf, false, 1))
    end
end

function play_audio_land()
    if not global_state.sfx_muted then
        root_scene:action(am.play(land_buf, false, 1))
    end
end

function play_audio_jump()
    if not global_state.sfx_muted then
        root_scene:action(am.play(jump_buf, false, 1))
    end
end

play_audio_menu_nav = play_audio_jump

local note_pitch = {
    --1,
    --2 ^ (2/12),
    --2 ^ (3/12),
    --2 ^ (5/12),
    --2 ^ (7/12),
    --2 ^ (8/12),
    --2 ^ (11/12),
    --2,
    1,
    2 ^ (1/12),
    2 ^ (2/12),
    2 ^ (3/12),
    2 ^ (4/12),
    2 ^ (5/12),
    2 ^ (6/12),
    2 ^ (7/12),
    2 ^ (8/12),
    2 ^ (9/12),
    2 ^ (10/12),
    2 ^ (11/12),
    2,
}

function play_audio_collect(note)
    if not global_state.sfx_muted then
        local pitch = note_pitch[math.clamp(note + 1, 1, #note_pitch)]
        --local pitch = 1
        root_scene:action(am.play(collect_buf, false, pitch))
    end
end

function play_audio_portal_open()
    if not global_state.sfx_muted then
        root_scene:action(am.play(portal_open_buf, false))
    end
end
