require "window"
require "level_visuals"
local sprites = require "sprites"

local stars_vshader = [[
    precision highp float;
    attribute vec3 pos;
    attribute vec2 uv;
    uniform mat4 MV;
    uniform mat4 P;
    uniform float t;
    uniform float depth;
    varying vec2 v_uv;
    void main() {
        v_uv = uv;
        float z = -fract(((t + pos.z) / depth)) * depth;
        vec3 final_pos = vec3(pos.xy, z);
        gl_Position = P * MV * vec4(final_pos, 1.0);
    }
]]

local stars_fshader = [[
    precision mediump float;
    uniform sampler2D tex;
    uniform vec3 color;
    varying vec2 v_uv;
    void main() {
        gl_FragColor = vec4(color * (texture2D(tex, v_uv).rgb), 1.0);
    }
]]

local stars_shader = am.program(stars_vshader, stars_fshader)

local stars 
local star_speed = -100

local
function make_stars()
    local aspect = win.width / win.height
    local num_stars = math.min(2000, 1000 * aspect)
    local quads = am.quads(num_stars, {"pos", "vec3", "uv", "vec2"})
    local rad = 100
    local depth = 200
    local w = 0.3
    local star_sprite = sprites.star
    local s1, t1, s2, t2 = star_sprite.s1, star_sprite.t1, star_sprite.s2, star_sprite.t2
    for i = 1, num_stars do
        local x = (math.random() * 2 - 1) * rad * aspect
        local y = (math.random() * 2 - 1) * rad
        local z = depth * math.random()
        quads:add_quad{
            pos = {x-w, y+w, z, x-w, y-w, z, x+w, y-w, z, x+w, y+w, z},
            uv = {s1, t2, s1, t1, s2, t1, s2, t2},
        }
    end
    local node = 
        am.bind{
            depth = depth,
            t = 0,
            P = math.perspective(math.rad(70), aspect, 10, depth),
            tex = sprites.texture,
            color = vec3(1),
        }
        ^ am.use_program(stars_shader)
        ^ am.blend"add"
        ^ quads

    local t = 0
    node:action(function(node)
        t = t + am.delta_time * star_speed
        node.t = t
    end)
    stars = node
end

make_stars()

--local galaxy_start_pos = vec3(300, 200, -2000)
--local galaxy_end_pos = vec3(0, 0, -150)
--local galaxy_sprite = am.sprite("bg_images/bg1.png")
--
--local
--function make_galaxy()
--    local rot1 = 
--        am.rotate(0)
--        ^ galaxy_sprite
--    local rot2 = 
--        am.rotate(0)
--        ^ am.scale(0.6)
--        ^ galaxy_sprite
--    local node = am.bind{
--            P = math.perspective(math.rad(30), win.width/win.height, 2, 4000),
--        } 
--        ^ am.translate(galaxy_end_pos)
--        ^ am.rotate(math.rad(105), vec3(1, 0, 0))
--        ^ am.scale(0.5)
--        ^ { rot1, rot2 }
--    node:action(function()
--        rot1.angle = am.frame_time * 0.1
--        rot2.angle = am.frame_time * 0.17
--    end)
--    return node
--end
--
--local galaxy = make_galaxy()

local node = am.group{
    stars,
}

node:action(am.series{
    am.delay(0.1),
    function()
        if win:resized() then
            node:remove_all()
            make_stars()
            node:append(stars)
        end
    end,
})

function set_bg_color(star_color, bg_color)
    win.clear_color = vec4(bg_color, 1)
    stars"bind".color = star_color
    --galaxy"sprite".color = vec4(star_color, 1)
end

function set_star_speed(speed)
    star_speed = -speed
end

function init_background(gs)
    show_background()
    gs.background_group = am.group()
    gs.background_group:append(node)
    gs.main_scene:append(gs.background_group)
    set_star_speed(50)
    local palette = gs.palette
    if gs.compound_level then
        -- use next level for background palette, which will only be revealed when transitioning to next level
        palette = get_level_palette(gs.level_num < num_levels and gs.level_num + 1 or 1)
        hide_background()
    end
    set_bg_color(palette[5], palette[4])
    reset_bg_speed()
    --galaxy"translate".position = math.mix(galaxy_start_pos, galaxy_end_pos, gs.level_num / 25)
end

function hide_background()
    node.hidden = true
end

function show_background()
    node.hidden = false
end

function get_bg()
    return node
end

function reset_bg_speed()
    set_star_speed(50)
end

local
function test()
    root_scene:remove_all()
    root_scene:append(stars)
end
