local palettes = require "palettes"

local limit = 10

local
function render_scores(mode, group, palette, x, y, rows, global)
    local dy = 22
    local default_color = vec4(palette[3], 1)
    local user_color = vec4(palette[6], 1)
    local x1 = x-220
    local x2 = x-210
    local x3 = x+230
    for i, row in ipairs(rows) do
        local rank = global and row.rank or i
        local color = row.is_user and user_color or default_color
        local rank_node = am.translate(x1, y) ^ am.scale(0.3) ^ am.text(font, rank, color, "right", "bottom")

        local name_node = am.translate(x2, y) ^ am.scale(0.3) ^ am.text(font, row.name, color, "left", "bottom")
        local score_txt
        if mode == "stage_speedrun" then
            score_txt = row.score and format_secs(row.score / 100) or ""
        else
            score_txt = row.score and tostring(row.score) or ""
        end
        local score_node = am.translate(x3, y) ^ am.scale(0.3) ^ am.text(font, score_txt, color, "right", "bottom")
        local node = am.group{
            rank_node, name_node, score_node,
        }
        group:append(node)
        y = y - dy
        if i >= limit then
            break
        end
    end
end

local mock_scores_friends = {
    {rank = 1, name = "Ian MacLarty", score = 30000},
    {rank = 2, name = "Bethany Wilksch", score = 30560},
    {rank = 3, name = "Yvonne McRory", score = 31560},
    {rank = 4, name = "faerwe4 McRory", score = 36780},
    {rank = 5, name = "withipthvonnt Mpfwjpe", score = 38880},
    {rank = 10, name = "godisrhog Mpfwjpe", score = 41000, is_user = true},
    {rank = 11, name = "efgsrklgh Mpfwjpe", score = 45000},
    {rank = 12, name = "fgsrdklgj Mpfwjpe", score = 46010},
    {rank = 13, name = "glsdrhg M;erjhge", score = 67000},
    {rank = 14, name = "gsrdiohose4gt Merjhiopg", score = 80240},
}

local mock_scores_global_top = {
    {rank = 1, name = "Ian MacLarty", score = 30000},
    {rank = 2, name = "Bethany Wilksch", score = 30561},
    {rank = 3, name = "Yvonne McRory", score = 31562},
    {rank = 4, name = "faerwe4 McRory", score = 36783},
    {rank = 5, name = "withipthvonnt Mpfwjpe", score = 38884},
    {rank = 10, name = "godisrhog Mpfwjpe", score = 41005},
    {rank = 11, name = "efgsrklgh Mpfwjpe", score = 45005},
    {rank = 12, name = "fgsrdklgj Mpfwjpe", score = 46016},
    {rank = 13, name = "glsdrhg M;erjhge", score = 67004},
    {rank = 14, name = "gsrdiohose4gt Merjhiopg", score = 80242},
}

local mock_scores_global_nbh = {
    {rank = 100, name = "fwlkehfoewhf", score = 90004},
    {rank = 101, name = "0g9eru0sgheg", score = 90563},
    {rank = 102, name = "fwe;khfoewhfgy", score = 91564},
    {rank = 103, name = "faerwe4 McRory", score = 96783},
    {rank = 104, name = "withipthvonnt Mpfwjpe", score = 98883, is_user = true},
    {rank = 105, name = "godisrhog Mpfwjpe", score = 141003},
    {rank = 106, name = "efgsrklgh Mpfwjpe", score = 145003},
    {rank = 107, name = "fgsrdklgj Mpfwjpe", score = 146013},
    {rank = 108, name = "glsdrhg M;erjhge", score = 167001},
    {rank = 109, name = "gsrdiohose4gt Merjhiopg", score = 180248},
}

local
function merge_scores(top, nbh)
    for i = 1, limit do
        if top[i] and top[i].is_user then
            return top
        end
    end
    local merged = {}
    for i = 1, limit/2 do
        if top[i] then
            table.insert(merged, top[i])
        end
    end
    table.insert(merged, {rank = "...", name = ""})
    local user_idx
    for i = 1, #nbh do
        if nbh[i] and nbh[i].is_user then
            user_idx = i
            break
        end
    end
    if not user_idx then
        return top
    end
    local start = math.max(1, user_idx - math.floor(limit / 4))
    local finish = #nbh
    for i = start, finish do
        if nbh[i] then
            table.insert(merged, nbh[i])
        end
    end
    return merged
end

function make_leaderboard(mode, stage)
    local palette = mode == "infinite" and palettes[1] or palettes[global_state.speedrun_stage or 1]
    local color = vec4(palette[3], 1)
    local scl = is_phone and 0.6 or 1
    local group = am.scale(scl)
    if am.steamworks_available then
        local header_y = 270
        local x1 = -260
        local x2 = 260
        local global_header = am.translate(x1, header_y)
            ^ am.scale(0.4) ^ am.text(font, strings.global_rank, color, "center", "top")
        local friends_header = am.translate(x2, header_y)
            ^ am.scale(0.4) ^ am.text(font, strings.friends_rank, color, "center", "top")
        group:append(global_header)
        group:append(friends_header)
        local scores_y = header_y - 80
        local global_top, global_nbh
        local 
        function global_top_leaderboard_cb(entries)
            if entries then
                global_top = entries 
                if global_nbh then
                    render_scores(mode, group, palette, x1, scores_y, merge_scores(global_top, global_nbh), true)
                end
            end
        end
        local 
        function global_nbh_leaderboard_cb(entries)
            if entries then
                global_nbh = entries
                if global_top then
                    render_scores(mode, group, palette, x1, scores_y, merge_scores(global_top, global_nbh), true)
                end
            end
        end
        local 
        function friends_leaderboard_cb(entries)
            render_scores(mode, group, palette, x2, scores_y, entries, false)
        end
        if mock_leaderboards then
            group:action(am.series{
                am.delay(0.5),
                function()
                    global_top_leaderboard_cb(mock_scores_global_top)
                    return true
                end,
                am.delay(0.1),
                function()
                    global_nbh_leaderboard_cb(mock_scores_global_nbh)
                    return true
                end,
                am.delay(0.1),
                function()
                    friends_leaderboard_cb(mock_scores_friends)
                    return true
                end,
            })
        elseif am.steamworks_available and am.steamworks_available() then
            local name
            if mode == "stage_speedrun" then
                name = "stage_speedrun_"..stage
            elseif mode == "infinite" then
                name = "infinite"
            else
                return
            end
            am.query_steamworks_leaderboard(name, "friends", 1, limit, friends_leaderboard_cb)
            am.query_steamworks_leaderboard(name, "global", 1, limit, global_top_leaderboard_cb)
            am.query_steamworks_leaderboard(name, "global_neighbourhood", -limit/2, limit/2, global_nbh_leaderboard_cb)
        end
    else
        -- not steam
        if mode == "stage_speedrun" then
            if global_state.best_times and global_state.best_times[stage] then
                local txt = strings.best_time..":  "..format_secs(global_state.best_times[stage])
                local y = win.safe_top / scl * 0.6
                group:append(
                    am.translate(0, y) 
                    ^ am.scale(0.4) 
                    ^ am.text(font, txt, color, "center", "top")
                )
            end
        end
    end

    return group
end
