require "window"
require "audio"
require "player"
local sprites = require "sprites"

local token_model = get_token_model()
local token_node = am.rotate(0, math.normalize(vec3(1, -2, 3))) ^ am.scale(vec3(8)) ^ token_model
token_node:action(function(node)
    node"rotate".angle = am.frame_time * 5
end)

function make_menu(items, mid_y, palette, selected)
    local bb_w = 500
    local dy = is_phone and 72 or is_tablet and 64 or 48
    local top = mid_y + dy * #items * 0.5
    local y = top
    local group = am.group()
    local on_color = vec4(palette[6], 1)
    local off_color = vec4(palette[3], 1)
    local curr = selected or 1
    local old_curr = selected or 1
    local widths = {}
    local bounding_boxes = {}
    local txt_scl = is_phone and 0.3 or 0.4
    local arrow_scl = is_mobile and txt_scl * 2 or txt_scl
    for i, item in ipairs(items) do
        local color = i == curr and not is_mobile and on_color or off_color
        local txt = item.text
        if type(txt) == "function" then
            txt = txt()
        end
        local scl = is_mobile and item.mobile_big and 1.5 * txt_scl or txt_scl
        local node = am.translate(0, y) ^ am.scale(scl, 0) ^ am.text(font, txt, color, "center", "center")
        widths[i] = node"text".width * scl
        if item.checkbox then
            local outer_sprite = sprites.checkbox_outer
            local inner_sprite = sprites.checkbox_inner
            local x_gap = 20
            local y_os = -3
            local w = outer_sprite.width * scl
            local h = outer_sprite.height * scl
            node.x = -(w + x_gap) * 0.5
            local checkbox_node = 
                am.translate((widths[i] * 0.5 + w * 0.5 + x_gap) / scl, y_os / scl) ^ {
                    am.sprite(outer_sprite, color, "center", "center"),
                    am.sprite(inner_sprite, color, "center", "center"):tag"checkbox_inner",
                }
            checkbox_node"checkbox_inner".hidden = not item.checkbox()
            node"scale":append(checkbox_node)
            widths[i] = widths[i] + w + x_gap
        elseif item.left and item.right then
            local arrow_sprite = sprites.ui_arrow_right
            local x_gap = 10
            local y_os = -3
            local w = arrow_sprite.width * arrow_scl
            local h = arrow_sprite.height * arrow_scl
            local right_arrow_node = 
                am.translate((widths[i] * 0.5 + w * 0.5 + x_gap) / scl, y_os / scl)
                ^ am.scale(arrow_scl / scl)
                ^ am.sprite(arrow_sprite, color, "center", "center")
            local left_arrow_node = 
                am.translate(-(widths[i] * 0.5 + w * 0.5 + x_gap) / scl, y_os / scl)
                ^ am.scale(-arrow_scl / scl, arrow_scl / scl)
                ^ am.sprite(arrow_sprite, color, "center", "center")
            node"scale":append(right_arrow_node)
            node"scale":append(left_arrow_node)
            widths[i] = widths[i] + 2 * (w + x_gap)
        end
        group:append(node)
        node"scale":action(am.tween(0.2, {y = scl}))
        if is_mobile and not global_state.less_flashing then
            node:action(function()
                local r = (math.sin(-am.frame_time * 5 + i) * 0.5 + 0.5) ^ 6
                local color = math.mix(off_color, on_color, r)
                node"text".color = color
                for _, spr in ipairs(node:all"sprite") do
                    spr.color = color
                end
            end)
        end
        bounding_boxes[i] = {
            x1 = -bb_w/2,
            x2 = bb_w/2,
            y1 = y - dy/2,
            y2 = y + dy/2,
        }
        y = y - dy
    end
    local token1 = am.translate(0, 0) ^ token_node
    local token2 = am.translate(0, 0) ^ token_node
    local tokens =
        am.bind{
            light_dir = math.normalize(vec3(1, 0, 0.75)),
            color = palette[3],
            shadow_color = palette[4],
            P = math.ortho(win.left, win.right, win.bottom, win.top, -100, 100),
        }
        ^ { token1, token2 }
    tokens:action(function()
        if win:resized() then
            tokens.P = math.ortho(win.left, win.right, win.bottom, win.top, -100, 100)
        end
    end)
    if not is_mobile then
        group:append(tokens)
    end
    local
    function set_tokens()
        local w = widths[curr]
        local y = top - (curr - 1) * dy - 2
        token1.x = -w * 0.5 - 30
        token2.x = w * 0.5 + 30
        token1.y = y
        token2.y = y
    end
    set_tokens()
    local
    function update_curr()
        group:child(old_curr):all"text".color = off_color
        group:child(old_curr):all"sprite".color = off_color
        if not is_mobile then
            group:child(curr):all"text".color = on_color
            group:child(curr):all"sprite".color = on_color
        end
        set_tokens()
        old_curr = curr
        play_audio_menu_nav()
    end
    local start_pos
    group:action(function()
        if is_mobile then
            if input_pressed() then
                start_pos = input_pos()
            elseif input_released() and start_pos then
                local pos = input_pos()
                local d = math.distance(pos, start_pos)
                if d < 20 then
                    for i, bb in ipairs(bounding_boxes) do
                        if pos.x >= bb.x1 and pos.x <= bb.x2 and pos.y >= bb.y1 and pos.y <= bb.y2 then
                            if items[i].action then
                                if items[i].action() then
                                    play_audio_menu_nav()
                                end
                            end
                            group:child(i)"text".text = items[i].text()
                            if items[i].checkbox then
                                group:child(i)"checkbox_inner".hidden = not items[i].checkbox()
                            end
                            if items[i].left and pos.x < 0 then
                                items[i].left()
                                play_audio_menu_nav()
                                group:child(i)"text".text = items[i].text()
                            end
                            if items[i].right and pos.x > 0 then
                                items[i].right()
                                play_audio_menu_nav()
                                group:child(i)"text".text = items[i].text()
                            end
                        end
                    end
                end
                start_pos = nil
            end
        else
            if down_pressed_ui() then
                curr = curr % #items + 1
                update_curr()
            elseif up_pressed_ui() then
                curr = (curr - 2) % #items + 1
                update_curr()
            elseif left_pressed_ui() then
                if items[curr].left then
                    items[curr].left()
                    play_audio_menu_nav()
                    group:child(curr)"text".text = items[curr].text()
                end
            elseif right_pressed_ui() then
                if items[curr].right then
                    items[curr].right()
                    play_audio_menu_nav()
                    group:child(curr)"text".text = items[curr].text()
                end
            elseif enter_pressed() then
                if items[curr].action then
                    if items[curr].action() then
                        play_audio_menu_nav()
                    end
                end
                group:child(curr)"text".text = items[curr].text()
                if items[curr].checkbox then
                    group:child(curr)"checkbox_inner".hidden = not items[curr].checkbox()
                end
            end
        end
    end)

    --[[
    if is_mobile then
        local top = bounding_boxes[1].y2
        local bottom = bounding_boxes[#bounding_boxes].y1
        group:prepend(am.rect(-bb_w/2, bottom, bb_w/2, top, vec4(0, 0, 0, 0.5)))
    end
    ]]
    return group
end
