require "settings"
require "background"
require "menu"
require "audio"
require "speedrun_menu"
require "about"
require "langs"
require "game"
require "infinite"
local sprites = require "sprites"

local title_y = is_mobile and 220 or 200

function get_title_img()
    local palette = get_menu_palette()
    local scl = is_mobile and 0.3 or 0.46
    local title_scale = math.min(scl, win.width / sprites.title.width * 0.9)
    local title = am.translate(0, title_y) ^ am.scale(title_scale) ^ {
        am.sprite(sprites.title, vec4(palette[1], 1)),
        am.sprite(sprites.title_outline, vec4(palette[6], 1)),
        am.translate(0, -300) ^ am.scale(0.65) ^ am.text(font, "by IAN MACLARTY", vec4(palette[6], 1)),
    }
    return title
end

function show_title()
    local palette = get_menu_palette()
    local lines = {}
    local grid_color1 = vec4(palette[3], 1)
    local grid_color2 = vec4(math.mix(palette[4], palette[3], 0.25), 1)
    local grid_delta = is_mobile and 50 or 100
    for y = -1200, 1200, grid_delta do
        if y > win.bottom and y < win.top then
            local line = am.line(vec2(-1200, y), vec2(-1200, y), 3, grid_color1)
            line:action(am.parallel{
                am.tween(1.8, {point2 = vec2(1200, y)}),
                am.series{
                    am.delay(0.8),
                    am.tween(1.0, {color = grid_color2}),
                }
            })
            table.insert(lines, line)
        end
    end
    for x = -1200, 1200, grid_delta do
        if x > win.left and x < win.right then
            local line = am.line(vec2(x, 1200), vec2(x, 1200), 3, grid_color1)
            line:action(am.parallel{
                am.tween(1.8, {point2 = vec2(x, -1200)}),
                am.series{
                    am.delay(0.8),
                    am.tween(1.0, {color = grid_color2}),
                }
            })
            table.insert(lines, line)
        end
    end
    local grid_overlay = am.group(lines)

    local title = get_title_img()

    local scene = am.group{
        grid_overlay,
        get_bg(),
        title,
    }

    if exhibition_mode then
        local press_start = am.translate(0, -120) ^ am.scale(0.5) ^ am.text(font, "PRESS START", vec4(palette[5], 1))
        scene:append(press_start)
        press_start:action(function()
            press_start.hidden = math.fract(am.frame_time) < 0.5
            if enter_pressed() then
                start_new_campaign()
            end
        end)
    else
        local menu_group
        local menu_y_mobile = (win.safe_bottom + title_y) * 0.5 - 65
        local main_menu_y = is_mobile and menu_y_mobile or -180
        local options_menu_y = is_mobile and menu_y_mobile or -180
        local new_game_confirm_menu_y = is_mobile and -200 or -220
        local new_game_confirm_y = is_mobile and 50 or 0

        local main_menu_items
        local options_menu_items
        local new_game_confirm_menu_items

        main_menu_items = {
            {
                text = function() return strings.new_game end,
                mobile_big = not global_state.saved_game,
                action = function()
                    if global_state.saved_game then
                        menu_group:remove_all()
                        local old_lh = font.line_height
                        font.line_height = 120
                        local scl = is_phone and 0.16 or is_tablet and 0.25 or 0.3
                        local confirm_text = 
                            am.translate(0, new_game_confirm_y)
                            ^ am.scale(scl)
                            ^ am.text(font, strings.new_game_confirm, vec4(palette[6], 1), "center", "left")
                        font.line_height = old_lh
                        local confirm_menu = am.group{
                            confirm_text,
                            make_menu(new_game_confirm_menu_items, new_game_confirm_menu_y, palette, 2)
                        }
                        menu_group:append(confirm_menu)
                        return true
                    else
                        start_new_campaign()
                        return true
                    end
                end,
            },
            {
                text = function() return strings.speedrun end,
                action = function()
                    show_speedrun_menu()
                    return true
                end,
            },
            {
                text = function() return strings.infinite end,
                action = function()
                    start_infinite_game(true)
                    return true
                end,
            },
            {
                text = function() return strings.options end,
                action = function()
                    menu_group:remove_all()
                    local options_menu = make_menu(options_menu_items, options_menu_y, palette, 1)
                    menu_group:append(options_menu)
                    return true
                end,
            },
            {
                text = function() return strings.about end,
                action = function()
                    show_about()
                    return true
                end,
            },
        }
        if not is_mobile then
            table.insert(main_menu_items,
                {
                    text = function() return strings.quit end,
                    action = function()
                        exit_game()
                    end,
                })
        end
        if global_state.saved_game then
            local prog = math.floor((global_state.saved_game.level-1) / num_levels * 100)
            table.insert(main_menu_items, 1, 
                {
                    text = function() return strings.continue.." ["..prog.."%]" end,
                    action = function()
                        continue_campaign()
                        return true
                    end,
                })
        end
        options_menu_items = {
            {
                text = function() return strings.sfx end,
                checkbox = function() return not global_state.sfx_muted end,
                action = function()
                    global_state.sfx_muted = not global_state.sfx_muted
                end,
            },
            {
                text = function() return strings.music end,
                checkbox = function() return not global_state.music_muted end,
                action = function()
                    global_state.music_muted = not global_state.music_muted
                    if global_state.music_muted then
                        stop_music()
                    else
                        start_menu_music()
                    end
                end,
            },
            {
                text = function() return strings.language end,
                right = function()
                    local lang = global_state.language or am.language()
                    local langs = get_supported_languages()
                    local l = table.search(langs, lang) or 1
                    l = l + 1
                    if l > #langs then
                        l = 1
                    end
                    global_state.language = langs[l]
                    if global_state.language == am.language() then
                        -- this allows changing language from the OS/steam/whatever,
                        -- even if we previosuly set the language in the game
                        global_state.language = nil
                    end
                    set_language()
                    menu_group:remove_all()
                    local options_menu = make_menu(options_menu_items, options_menu_y, palette, 4)
                    menu_group:append(options_menu)
                end,
                left = function()
                    local lang = global_state.language or am.language()
                    local langs = get_supported_languages()
                    local l = table.search(langs, lang) or 1
                    l = l - 1
                    if l < 1 then
                        l = #langs
                    end
                    global_state.language = langs[l]
                    set_language()
                    menu_group:remove_all()
                    local options_menu = make_menu(options_menu_items, options_menu_y, palette, 4)
                    menu_group:append(options_menu)
                end
            },
            {
                text = function() 
                    return string.format("%s %d %%", strings.game_speed, global_state.game_speed or 100)
                end,
                right = function()
                    global_state.game_speed = math.clamp((global_state.game_speed or 100) + 5, 10, 100)
                end,
                left = function()
                    global_state.game_speed = math.clamp((global_state.game_speed or 100) - 5, 10, 100)
                end,
            },
            {
                text = function() 
                    return strings.less_flashing
                end,
                checkbox = function() return global_state.less_flashing end,
                action = function()
                    global_state.less_flashing = not global_state.less_flashing
                    if is_mobile then
                        menu_group:remove_all()
                        local options_menu = make_menu(options_menu_items, options_menu_y, palette, 4)
                        menu_group:append(options_menu)
                    end
                    return true
                end,
            },
            {
                text = function() return strings.back end,
                action = function()
                    menu_group:remove_all()
                    local main_menu = make_menu(main_menu_items, main_menu_y, palette, 1)
                    menu_group:append(main_menu)
                    save_state()
                    return true
                end
            },
        }
        if not is_mobile then
            table.insert(options_menu_items, 3,
            {
                text = function() return strings.fullscreen end,
                checkbox = function() return global_state.fullscreen end,
                action = function()
                    global_state.fullscreen = not global_state.fullscreen
                    win.mode = global_state.fullscreen and "fullscreen" or "windowed"
                    win.show_cursor = not global_state.fullscreen
                    return true
                end,
            })
        end
        new_game_confirm_menu_items = {
            {
                text = function() return strings.yes end,
                action = function()
                    start_new_campaign()
                    return true
                end,
            },
            {
                text = function() return strings.no end,
                action = function()
                    menu_group:remove_all()
                    local main_menu = make_menu(main_menu_items, main_menu_y, palette, 1)
                    menu_group:append(main_menu)
                    return true
                end,
            },
        }
        local main_menu = make_menu(main_menu_items, main_menu_y, palette, 1)
        menu_group = am.group{main_menu}
        scene:append(menu_group)
    end
    show_background()
    root_scene:remove_all()
    root_scene:append(scene)
    reset_bg_speed()
    set_bg_color(palette[5], palette[4])
    start_menu_music()
end
