local sprites = require "sprites"

function show_tutorial(gs)
    local color = vec4(gs.palette[2], 1)
    local
    function get_pos()
        if win.width > win.height then
            return vec2(gs.play_area_width*0.5 - 80, -40)
        else
            return vec2(40, 100 - gs.play_area_height*0.5)
        end
    end
    if is_mobile then
        local cubes0 = global_state.total_cubes_collected or 0
        local transparent = vec4(color.rgb, 0)
        local delta = 30
        local hand = am.translate(get_pos())
            ^ am.scale(0.35) 
            ^ am.sprite(sprites.hand, color)
        local swipe_scl = 0.5
        local swipe = am.translate(get_pos())
            ^ am.rotate(0)
            ^ am.scale(0, swipe_scl) 
            ^ am.sprite(sprites.swipe, color)
        local instruction = am.translate(0, 140)
            ^ am.scale(0.12)
            ^ {
                am.rect(-100, -60, 100, 50, vec4(0, 0, 0, 0.2)),
                am.text(font, strings.touch_instructions1, color)
            }
        if gs.level.name == "ACS" then
            instruction"text".text = strings.jump_tutorial_prompt
        end
        instruction:action(function()
            instruction"text".color = math.mix(color, transparent, (math.sin(gs.real_time * 5) * 0.5 + 0.5) ^ 2 * 0.5)
            if gs.mode == "infinite" then
                if  strings.inf_instructions2 ~= "" then
                    instruction"text".text = math.fract(gs.real_time * 0.2) < 0.5 and strings.touch_instructions1 or strings.inf_instructions2
                end
            elseif gs.level.name ~= "ACS" then
                instruction"text".text = math.fract(gs.real_time * 0.2) < 0.5 and strings.touch_instructions1 or strings.touch_instructions2
            end
            local rect = instruction"rect"
            local w = instruction"text".width
            local h = instruction"text".height
            rect.x1 = -w/2 - 50
            rect.x2 = w/2 + 50
            rect.y1 = -h/2 - 45
            rect.y2 = h/2 + 30
            if gs.level.name == "ACS" and strings.jump_tutorial_prompt == "" then
                rect.hidden = true
            end
        end)
        if gs.mode == "infinite" and not is_phone then
            gs.tutorial_node = am.rotate(math.rad(-90)) ^ {swipe, hand, instruction}
        elseif gs.level.name == "ACS" then
            gs.tutorial_node = am.group{instruction}
        else
            gs.tutorial_node = am.group{swipe, hand, instruction}
        end
        local hand_sprite = hand"sprite"
        local swipe_sprite = swipe"sprite"
        local swipe_scale = swipe"scale"
        local swipe_rotate = swipe"rotate"
        hand_sprite.color = transparent
        swipe_sprite.color = transparent
        gs.tutorial_node:action("tut", coroutine.create(function()
            while true do
                hand.position2d = get_pos() - vec2(delta, 0)
                swipe_rotate.angle = 0
                swipe_scale.scale2d = vec2(0, swipe_scl)
                am.wait(
                    am.parallel{
                        am.series{
                            am.tween(hand_sprite, 0.1, {color = color}),
                            am.tween(hand, 0.3, {position2d = get_pos() + vec2(delta, 0)}, am.ease.out(am.ease.cubic)),
                            am.tween(hand_sprite, 0.2, {color = transparent}),
                        },
                        am.series{
                            am.parallel{
                                am.tween(swipe_sprite, 0.3, {color = color}),
                                am.tween(swipe_scale, 0.4, {scale2d = vec2(swipe_scl)}),
                            },
                            am.tween(swipe_sprite, 0.1, {color = transparent}),
                        },
                    }
                )
                am.wait(am.delay(0.4))

                hand.position2d = get_pos() - vec2(0, -delta)
                swipe_rotate.angle = math.rad(-90)
                swipe_scale.scale2d = vec2(0, swipe_scl)
                am.wait(
                    am.parallel{
                        am.series{
                            am.tween(hand_sprite, 0.1, {color = color}),
                            am.tween(hand, 0.3, {position2d = get_pos() + vec2(0, -delta)}, am.ease.out(am.ease.cubic)),
                            am.tween(hand_sprite, 0.2, {color = transparent}),
                        },
                        am.series{
                            am.parallel{
                                am.tween(swipe_sprite, 0.3, {color = color}),
                                am.tween(swipe_scale, 0.4, {scale2d = vec2(swipe_scl)}),
                            },
                            am.tween(swipe_sprite, 0.1, {color = transparent}),
                        },
                    }
                )
                am.wait(am.delay(0.4))

                hand.position2d = get_pos() - vec2(-delta, 0)
                swipe_rotate.angle = math.rad(180)
                swipe_scale.scale2d = vec2(0, swipe_scl)
                am.wait(
                    am.parallel{
                        am.series{
                            am.tween(hand_sprite, 0.1, {color = color}),
                            am.tween(hand, 0.3, {position2d = get_pos() + vec2(-delta, 0)}, am.ease.out(am.ease.cubic)),
                            am.tween(hand_sprite, 0.2, {color = transparent}),
                        },
                        am.series{
                            am.parallel{
                                am.tween(swipe_sprite, 0.3, {color = color}),
                                am.tween(swipe_scale, 0.4, {scale2d = vec2(swipe_scl)}),
                            },
                            am.tween(swipe_sprite, 0.1, {color = transparent}),
                        },
                    }
                )
                am.wait(am.delay(0.4))

                hand.position2d = get_pos() - vec2(0, delta)
                swipe_rotate.angle = math.rad(90)
                swipe_scale.scale2d = vec2(0, swipe_scl)
                am.wait(
                    am.parallel{
                        am.series{
                            am.tween(hand_sprite, 0.1, {color = color}),
                            am.tween(hand, 0.3, {position2d = get_pos() + vec2(0, delta)}, am.ease.out(am.ease.cubic)),
                            am.tween(hand_sprite, 0.2, {color = transparent}),
                        },
                        am.series{
                            am.parallel{
                                am.tween(swipe_sprite, 0.3, {color = color}),
                                am.tween(swipe_scale, 0.4, {scale2d = vec2(swipe_scl)}),
                            },
                            am.tween(swipe_sprite, 0.1, {color = transparent}),
                        },
                    }
                )
                am.wait(am.delay(0.4))
            end
        end))
        if gs.mode == "infinite" then
            gs.tutorial_node:action(function()
                local cubes = global_state.total_cubes_collected or 0
                if cubes - cubes0 >= 3 then
                    gs.main_scene:remove(gs.tutorial_node)
                    gs.tutorial_node = nil
                    return true
                end
            end)
        end
    else
        local border = am.sprite(sprites.button_border, color)
        local y_os = 5
        local arrow_key = am.group{border, am.sprite(sprites.arrow_up, color)}
        local arrow_keys = am.translate(get_pos()) ^ am.scale(0.25) ^ {
            am.translate(0, 0):tag"button_up" ^ arrow_key,
            am.translate(-100, -100):tag"button_left" ^ am.rotate(math.rad(90)) ^ arrow_key,
            am.translate(100, -100):tag"button_right" ^ am.rotate(math.rad(-90)) ^ arrow_key,
            am.translate(0, -100):tag"button_down" ^ am.rotate(math.rad(180)) ^ arrow_key,
        }
        local wasd_keys = am.translate(get_pos()) ^ am.scale(0.25) ^ {
            am.translate(0, 0):tag"button_up" ^ am.group{border, am.scale(0.6) ^ am.translate(0, y_os) ^ am.text(font, "W", color)},
            am.translate(-100, -100):tag"button_left" ^ am.group{border, am.scale(0.6) ^ am.translate(0, y_os) ^ am.text(font, "A", color)},
            am.translate(100, -100):tag"button_right" ^ am.group{border, am.scale(0.6) ^ am.translate(0, y_os) ^ am.text(font, "D", color)},
            am.translate(0, -100):tag"button_down" ^ am.group{border, am.scale(0.6) ^ am.translate(0, y_os) ^ am.text(font, "S", color)},
        }
        local a_color = vec4(gs.palette[16], 1)
        local b_color = vec4(gs.palette[12], 1)
        local x_color = vec4(gs.palette[14], 1)
        local y_color = vec4(gs.palette[11], 1)
        local abxy_buttons = am.translate(get_pos()) ^ am.scale(0.25) ^ {
            am.translate(0, 50):tag"button_up" ^ am.group{am.sprite(sprites.controller_button_border, y_color), am.scale(0.5) 
                ^ am.translate(0, y_os) ^ am.text(font, "Y", y_color)},
            am.translate(-100, -50):tag"button_left" ^ am.group{am.sprite(sprites.controller_button_border, x_color), am.scale(0.5) 
                ^ am.translate(0, y_os) ^ am.text(font, "X", x_color)},
            am.translate(100, -50):tag"button_right" ^ am.group{am.sprite(sprites.controller_button_border, b_color), am.scale(0.5) 
                ^ am.translate(0, y_os) ^ am.text(font, "B", b_color)},
            am.translate(0, -150):tag"button_down" ^ am.group{am.sprite(sprites.controller_button_border, a_color), am.scale(0.5) 
                ^ am.translate(0, y_os) ^ am.text(font, "A", a_color)},
        }
        local nodes = {arrow_keys, wasd_keys}
        if am.controller_present(1) then
            table.insert(nodes, 1, abxy_buttons)
        end
        gs.tutorial_node = am.group()
        gs.tutorial_node:action(coroutine.create(function()
            while true do
                local i = 1
                while i <= #nodes do
                    gs.tutorial_node:remove_all()
                    gs.tutorial_node:append(nodes[i])
                    am.wait(am.delay(0.2))
                    nodes[i]"button_up".hidden = true
                    am.wait(am.delay(0.15))
                    nodes[i]"button_up".hidden = false
                    am.wait(am.delay(0.2))
                    nodes[i]"button_right".hidden = true
                    am.wait(am.delay(0.15))
                    nodes[i]"button_right".hidden = false
                    am.wait(am.delay(0.2))
                    nodes[i]"button_down".hidden = true
                    am.wait(am.delay(0.15))
                    nodes[i]"button_down".hidden = false
                    am.wait(am.delay(0.2))
                    nodes[i]"button_left".hidden = true
                    am.wait(am.delay(0.15))
                    nodes[i]"button_left".hidden = false
                    am.wait(am.delay(0.2))
                    i = i + 1
                end
            end
        end))
    end
    gs.main_scene:append(gs.tutorial_node)
end

