require "save"

is_mobile = am.platform == "ios" or am.platform == "android"
is_really_mobile = is_mobile

local mobile_w = 380
local mobile_h = 650

local width, height = 1200, 700
if is_editor then
    width = 1900
    height = 1200
end

if is_mobile then
    width = mobile_w
    height = mobile_h
end

local fullscreen = global_state.fullscreen
if table.search(arg, "-f") then
    fullscreen = true
elseif table.search(arg, "-w") then
    fullscreen = false
end
table.remove_all(arg, "-f")
table.remove_all(arg, "-w")

local emulate_iphone = table.search(arg, "-iphone")
table.remove_all(arg, "-iphone")
if emulate_iphone then
    width = mobile_w
    --height = 16/9*width
    height = 19.5/9*width
    is_mobile = true
end

local emulate_ipad = table.search(arg, "-ipad")
table.remove_all(arg, "-ipad")
if emulate_ipad then
    height = mobile_h
    width = 4/3*height
    is_mobile = true
end

mock_leaderboards = false
if table.search(arg, "-mock-leaderboards") then
    table.remove_all(arg, "-mock-leaderboards")
    mock_leaderboards = true
end

capture_mode = false
if table.search(arg, "-capture") then
    table.remove_all(arg, "-capture")
    --width = 886/2.3
    --height = 1920/2.3
    --is_mobile = true
    width = 1920
    height = 1080
    capture_mode = true
end

screenshot_mode = false
if table.search(arg, "-screenshot-mode") then
    table.remove_all(arg, "-screenshot-mode")
    screenshot_mode = true
    --width = 1920
    --height = 1080
    width = 800
    height = 600
    is_mobile = true
end

exhibition_mode = false
if table.search(arg, "-exhibition") then
    table.remove_all(arg, "-exhibition")
    exhibition_mode = true
    fullscreen = true
end

win = am.window{
    title = "JUMPGRID",
    letterbox = false,
    show_cursor = not fullscreen or is_editor,
    width = width,
    height = height,
    mode = fullscreen and "fullscreen" or "windowed",
    highdpi = true,
    msaa_samples = screenshot_mode and 4 or nil,
    stencil_buffer = true,
    depth_buffer = true,
    orientation = "hybrid",
}

--am.show_perf_overlay()

if is_mobile then
    if win.width > win.height then
        is_tablet = true
        is_phone = false
    else
        is_tablet = false
        is_phone = true
    end
else
    is_tablet = false
    is_phone = false
end

require "i18n"

root_scene = am.depth_test("always", false)

win.scene = root_scene

local debug_text
local debug_lines = {}
local max_lines = 10

local max_touch_deltas = 5
local recent_touch_deltas = {{}, {}}
local move_start = {nil, nil}
--local did_move = {false, false}
local last_nolift_touch_dir = {false, false}
local touch_move_count = {0, 0}
local max_touch_moves = 2
local move_nolift_threshold = 50
local move_lift_threshold = 10
local min_move_gap_time = 0.1
local time_since_last_move = {0, 0}
local left_axis_x = 0
local left_axis_y = 0
local right_axis_x = 0
local right_axis_y = 0
local was_left_tap = false
local was_right_tap = false
local was_up_tap = false
local was_down_tap = false
local need_left_lreset = false
local need_right_lreset = false
local need_up_lreset = false
local need_down_lreset = false

root_scene:action(function()
    for i = 1, 2 do
        -- if win:touch_began(i) then
        --     move_start[i] = win:touch_pixel_position(i)
        --     --last_nolift_touch_dir[i] = false
        -- end
        -- --if time_since_last_move[i] < min_lift_gap_time and (win:touch_active(i) or win:touch_ended(i)) then
        -- --    move_start[i] = win:touch_pixel_position(i)
        -- --end
        -- --time_since_last_lift[i] = time_since_last_lift[i] + am.delta_time
        if win:touch_began(i) then
            recent_touch_deltas[i] = {}
            move_start[i] = win:touch_pixel_position(i)
            --did_move[i] = false
            last_nolift_touch_dir[i] = false
            touch_move_count[i] = 0
        end
        if win:touch_active(i) or win:touch_ended(i) then
            table.insert(recent_touch_deltas[i], {d = win:touch_pixel_delta(i), t = am.frame_time})
            while #recent_touch_deltas[i] > max_touch_deltas do
                table.remove(recent_touch_deltas[i], 1)
            end
        end
        time_since_last_move[i] = time_since_last_move[i] + am.delta_time
    end

    if am.controller_present(1) then
        was_left_tap = false
        was_right_tap = false
        was_up_tap = false
        was_down_tap = false
        local lpos = am.controller_lstick_pos(1)
        local new_left_axis_x = lpos.x
        local new_left_axis_y = lpos.y
        local dead_zone = 0.4
        if need_left_lreset and new_left_axis_x > -dead_zone then
            need_left_lreset = false
        end
        if need_right_lreset and new_left_axis_x < dead_zone then
            need_right_lreset = false
        end
        if need_up_lreset and new_left_axis_y < dead_zone then
            need_up_lreset = false
        end
        if need_down_lreset and new_left_axis_y > -dead_zone then
            need_down_lreset = false
        end
        if new_left_axis_x < -0.6 and not need_left_lreset then
            was_left_tap = true
            need_left_lreset = true
        end
        if new_left_axis_x > 0.6 and not need_right_lreset then
            was_right_tap = true
            need_right_lreset = true
        end
        if new_left_axis_y < -0.6 and not need_down_lreset then
            was_down_tap = true
            need_down_lreset = true
        end
        if new_left_axis_y > 0.6 and not need_up_lreset then
            was_up_tap = true
            need_up_lreset = true
        end
        left_axis_x = new_left_axis_x
        left_axis_y = new_left_axis_y
    end
end)

function input_pos()
    if is_really_mobile then
        return win:touch_position(1)
    else
        return win:mouse_position()
    end
end

function input_pos_norm()
    if is_really_mobile then
        return win:touch_norm_position(1)
    else
        return win:mouse_norm_position()
    end
end

function input_delta()
    if is_really_mobile then
        return win:touch_delta(1)
    else
        return win:mouse_delta()
    end
end

function input_down()
    if is_really_mobile then
        return win:touch_active(1)
    else
        return win:mouse_down"left" or win:key_down"z"
    end
end

function input_released()
    if is_really_mobile then
        return win:touch_ended(1)
    else
        return win:mouse_released"left" or win:key_released"z"
    end
end

function input_pressed()
    if is_really_mobile then
        return win:touch_began(1)
    else
        return win:mouse_pressed"left" or win:key_pressed"z"
    end
end

function input_pressed2()
    if is_really_mobile then
        return win:touch_began(2)
    else
        return win:mouse_pressed"right" or win:key_pressed"x"
    end
end

local
function get_dir(d)
    local ax = math.abs(d.x)
    local ay = math.abs(d.y)
    if ax > ay then
        if d.x > 0 then
            return "right"
        else
            return "left"
        end
    elseif ay > ax then
        if d.y > 0 then
            return "up"
        else
            return "down"
        end
    end
end

function left_pressed()
    return win:key_pressed"left" or win:key_pressed"a" or am.controller_button_pressed(1, "left") or am.controller_button_pressed(1, "x") or was_left_tap
end

function right_pressed()
    return win:key_pressed"right" or win:key_pressed"d" or am.controller_button_pressed(1, "right") or am.controller_button_pressed(1, "b") or was_right_tap
end

function up_pressed()
    return win:key_pressed"up" or win:key_pressed"w" or am.controller_button_pressed(1, "up") or am.controller_button_pressed(1, "y") or was_up_tap
end

function down_pressed()
    return win:key_pressed"down" or win:key_pressed"s" or am.controller_button_pressed(1, "down") or am.controller_button_pressed(1, "a") or was_down_tap
end

function left_pressed_ui()
    return win:key_pressed"left" or win:key_pressed"a" or am.controller_button_pressed(1, "left") or was_left_tap
end

function right_pressed_ui()
    return win:key_pressed"right" or win:key_pressed"d" or am.controller_button_pressed(1, "right") or was_right_tap
end

function up_pressed_ui()
    return win:key_pressed"up" or win:key_pressed"w" or am.controller_button_pressed(1, "up") or was_up_tap
end

function down_pressed_ui()
    return win:key_pressed"down" or win:key_pressed"s" or am.controller_button_pressed(1, "down") or was_down_tap
end

function enter_pressed()
    return win:key_pressed"enter" or win:key_pressed"space" or am.controller_button_pressed(1, "a") or am.controller_button_pressed(1, "x")
end

function esc_pressed()
    return win:key_pressed"escape" or am.controller_button_pressed(1, "start")
end

last_movement_input_time = 0

local
function compute_touch_delta(i)
    local n = #recent_touch_deltas[i]
    local d = vec2(0)
    for j = 1, n do
        d = d + recent_touch_deltas[i][j].d
    end
    return d
end

function input_movement()
    local dir
    if is_really_mobile then
        -- for i = 1, 2 do
        --     if win:touch_ended(i) then
        --         local move = win:touch_pixel_position(i) - move_start[i]
        --         if math.length(move) > move_lift_threshold then
        --             dir = get_dir(move)
        --             --[[
        --             if last_nolift_touch_dir[i] ~= dr and time_since_last_lift[i] >= min_lift_gap_time then
        --                 dir = dr
        --                 log("lift move: %f %f", math.length(move), time_since_last_lift[i])
        --             else
        --                 if last_nolift_touch_dir[i] == dr then
        --                     log("ignored lift, because dir same: %f %f", math.length(move), time_since_last_lift[i])
        --                 else
        --                     log("ignored lift, because too quick: %f %f", math.length(move), time_since_last_lift[i])
        --                 end
        --             end
        --             ]]
        --         --else
        --         --    log("ignored lift, because too short: %f %f", math.length(move), time_since_last_lift[i])
        --         end
        --         --time_since_last_lift[i] = 0
        --     --elseif move_start[i] and win:touch_active(i) then
        --     --    local pos = win:touch_pixel_position(i)
        --     --    local move = pos - move_start[i]
        --     --    local d = math.length(move)
        --     --    if d > move_nolift_threshold then
        --     --        local dr = get_dir(move)
        --     --        if last_nolift_touch_dir[i] ~= dr then
        --     --            dir = dr
        --     --            last_nolift_touch_dir[i] = dr
        --     --            move_start[i] = pos
        --     --            log("no-lift move: %f %f", math.length(move), time_since_last_lift[i])
        --     --        else
        --     --            log("ignored no-lift move, because same dir: %f %f", math.length(move), time_since_last_lift[i])
        --     --        end
        --     --    end
        --     end
        -- end
        -- --if win:touch_began(1) then
        -- --    local pos = win:touch_norm_position()
        -- --    if pos.y < 0 then
        -- --        dir = get_dir(pos + vec2(0, 0.5))
        -- --    end
        -- --end
        for i = 1, 2 do
            if touch_move_count[i] < max_touch_moves and time_since_last_move[i] >= min_move_gap_time then
                if win:touch_ended(i) then
                    local delta = compute_touch_delta(i)
                    local l = math.length(delta)
                    if not last_nolift_touch_dir[i] and l > move_lift_threshold or l > move_nolift_threshold then
                        local dr = get_dir(delta)
                        if dr ~= last_nolift_touch_dir[i] then
                            dir = dr
                            touch_move_count[i] = touch_move_count[i] + 1
                            time_since_last_move[i] = 0
                        end
                    end
                elseif move_start[i] and win:touch_active(i) then
                    local delta = compute_touch_delta(i)
                    if math.length(delta) > move_nolift_threshold then
                        local dr = get_dir(delta)
                        if dr ~= last_nolift_touch_dir[i] then
                            dir = dr
                            last_nolift_touch_dir[i] = dir
                            recent_touch_deltas[i] = {}
                            touch_move_count[i] = touch_move_count[i] + 1
                            time_since_last_move[i] = 0
                        end
                    end
                end
            end
        end
    else
        if left_pressed() then
            dir = "left"
        elseif right_pressed() then
            dir = "right"
        elseif up_pressed() then
            dir = "up"
        elseif down_pressed() then
            dir = "down"
        end
    end
    if dir then
        last_movement_input_time = am.frame_time
    end
    return dir
end

function exit_game()
    win:close()
end
